/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/*
 *  File-Roller
 *
 *  Copyright (C) 2001-2009 The Free Software Foundation, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>
#include <string.h>
#include <glib/gi18n.h>
#include <gdk/gdkx.h>
#include <gtk/gtk.h>
#include "dlg-package-installer.h"
#include "gio-utils.h"
#include "glib-utils.h"
#include "gtk-utils.h"
#include "fr-init.h"


#define BUFFER_SIZE_FOR_PRELOAD 32

GtkWidget *warning_dialog,*install_lexicon_progress_bar,*install_lexicon_label;
int installWhitelistItemOver = 0;
int InstallSuccessOrNot;
gint RefreshInstallItemId;
typedef struct {
	FrWindow     *window;
	FrAction      action;
	GCancellable *cancellable;
	const char   *packages;
} InstallerData;

void InstallLexiconListThread();

void InstallLexiconListProcessShow(GtkWidget  *parent);
gboolean refresh_install_item(GtkWidget *parent);
	
	static void
installer_data_free (InstallerData *idata)
{
	g_object_unref (idata->window);
	_g_object_ref (idata->cancellable);
	g_free (idata);
}


static void
package_installer_terminated (InstallerData *idata,
			      FrErrorType    error_type,
			      const char    *error_message)
{
	GdkWindow *window;

	window = gtk_widget_get_window (GTK_WIDGET (idata->window));
	if (window != NULL)
		gdk_window_set_cursor (window, NULL);

	if (error_type != FR_ERROR_NONE) {
		fr_window_stop_batch_with_error (idata->window,
						 idata->action,
						 error_type,
						 error_message);
	}
	else {
		update_registered_archives_capabilities ();
		if (fr_window_is_batch_mode (idata->window))
			fr_window_resume_batch (idata->window);
		else
			fr_window_restart_current_batch_action (idata->window);
	}

	installer_data_free (idata);
}


#ifdef ENABLE_PACKAGEKIT


static void
packagekit_install_package_names_ready_cb (GObject      *source_object,
					   GAsyncResult *res,
					   gpointer      user_data)
{
	InstallerData   *idata = user_data;
	GDBusProxy      *proxy;
	GVariant        *values;
	GError          *error = NULL;
	FrErrorType  error_type = FR_ERROR_NONE;
	char            *error_message = NULL;

	proxy = G_DBUS_PROXY (source_object);
	values = g_dbus_proxy_call_finish (proxy, res, &error);
	if (values == NULL) {
		if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)
		    || (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_DBUS_ERROR)
			&& (error->message != NULL)
			&& (strstr (error->message, "org.freedesktop.Packagekit.Modify.Cancelled") != NULL)))
		{
			error_type = FR_ERROR_STOPPED;
			error_message = NULL;
		}
		else {
			error_type = FR_ERROR_GENERIC;
			error_message = g_strdup_printf ("%s\n%s",
							 _("There was an internal error trying to search for applications:"),
							 error->message);
		}
		g_clear_error (&error);
	}

	package_installer_terminated (idata, error_type, error_message);

	g_free (error_message);
	if (values != NULL)
		g_variant_unref (values);
	g_object_unref (proxy);
}


static char **
get_packages_real_names (char **names)
{
	char     **real_names;
	GKeyFile  *key_file;
	char      *filename;
	int        i;

	real_names = g_new0 (char *, g_strv_length (names));
	key_file = g_key_file_new ();
	filename = g_build_filename (PRIVDATADIR, "packages.match", NULL);
	g_key_file_load_from_file (key_file, filename, G_KEY_FILE_NONE, NULL);

	for (i = 0; names[i] != NULL; i++) {
		char *real_name;

		real_name = g_key_file_get_string (key_file, "Package Matches", names[i], NULL);
		if (real_name != NULL)
			real_name = g_strstrip (real_name);
		if ((real_name == NULL) || (strncmp (real_name, "", 1) == 0)) {
			g_free (real_name);
			real_name = g_strdup (names[i]);
		}
		real_names[i] = real_name;
		real_name = NULL;
	}

	g_free (filename);
	g_key_file_free (key_file);

	return real_names;
}


static void
install_packages (InstallerData *idata)
{
	GDBusConnection *connection;
	GError          *error = NULL;

	connection = g_bus_get_sync (G_BUS_TYPE_SESSION, idata->cancellable, &error);
	if (connection != NULL) {
		GdkWindow  *window;
		GDBusProxy *proxy;

		window = gtk_widget_get_window (GTK_WIDGET (idata->window));
		if (window != NULL) {
			GdkCursor *cursor;

			cursor = gdk_cursor_new (GDK_WATCH);
			gdk_window_set_cursor (window, cursor);

			g_object_unref (cursor);
		}

		proxy = g_dbus_proxy_new_sync (connection,
					       G_DBUS_PROXY_FLAGS_NONE,
					       NULL,
					       "org.freedesktop.PackageKit",
					       "/org/freedesktop/PackageKit",
					       "org.freedesktop.PackageKit.Modify",
					       idata->cancellable,
					       &error);

		if (proxy != NULL) {
			guint   xid;
			char  **names;
			char  **real_names;

			if (window != NULL)
				xid = GDK_WINDOW_XID (window);
			else
				xid = 0;

			names = g_strsplit (idata->packages, ",", -1);
			real_names = get_packages_real_names (names);

			g_dbus_proxy_call (proxy,
					   "InstallPackageNames",
					   g_variant_new ("(u^ass)",
							  xid,
							  real_names,
							  "hide-confirm-search,hide-finished,hide-warning"),
					   G_DBUS_CALL_FLAGS_NONE,
					   G_MAXINT,
					   idata->cancellable,
					   packagekit_install_package_names_ready_cb,
					   idata);

			g_strfreev (real_names);
			g_strfreev (names);
		}
	}

	if (error != NULL) {
		char *message;

		message = g_strdup_printf ("%s\n%s",
					   _("There was an internal error trying to search for applications:"),
					   error->message);
		package_installer_terminated (idata, FR_ERROR_GENERIC, message);

		g_clear_error (&error);
	}
}


static void
confirm_search_dialog_response_cb (GtkDialog *dialog,
				   int        response_id,
				   gpointer   user_data)
{
	InstallerData *idata = user_data;

	gtk_widget_destroy (GTK_WIDGET (dialog));

	if (response_id == GTK_RESPONSE_YES) {
		install_packages (idata);
	}
	else {
		fr_window_stop_batch (idata->window);
		installer_data_free (idata);
	}
}


#endif /* ENABLE_PACKAGEKIT */
void show_warning_dialog(GtkWidget *parent)
{

	GtkWidget *install_lexicon_progress_layout,*install_lexicon_progress_value_label;
	warning_dialog = gtk_window_new(GTK_WINDOW_TOPLEVEL);
        gtk_window_set_title(warning_dialog ,_("Installing"));
        gtk_window_set_position(GTK_WINDOW( warning_dialog),GTK_WIN_POS_CENTER);
        gtk_window_set_default_size(GTK_WINDOW(warning_dialog),310,200);
        gtk_window_set_modal(warning_dialog,TRUE);
        gtk_window_set_resizable(GTK_WINDOW (warning_dialog), FALSE);
        gtk_window_set_transient_for(warning_dialog,parent);
        gtk_window_set_decorated(GTK_WINDOW(warning_dialog), FALSE);
        install_lexicon_progress_layout = gtk_layout_new(NULL, NULL);

        install_lexicon_label = gtk_label_new(_("check and install,please wait for moment...."));
        gtk_widget_show(install_lexicon_label);
        install_lexicon_progress_layout = gtk_layout_new(NULL, NULL);
        gtk_widget_show(install_lexicon_progress_layout);
        gtk_widget_set_size_request(install_lexicon_progress_layout,310,190);
        install_lexicon_progress_bar = gtk_progress_bar_new();
        gtk_widget_show(install_lexicon_progress_bar);
        gtk_widget_set_size_request(install_lexicon_progress_bar, 200, 20);
        gtk_layout_put(GTK_LAYOUT(install_lexicon_progress_layout),install_lexicon_label,70,70);


        gtk_container_add(GTK_CONTAINER(warning_dialog),install_lexicon_progress_layout);
        gtk_widget_show(warning_dialog);

	refresh_install_lexicon_item_thread(parent);

        install_lexicon_list_thread();



}
void refresh_install_lexicon_item_thread(GtkWidget  *parent)
{


    g_thread_new("install_lexicon_process",InstallLexiconListProcessShow,parent);
}
void InstallLexiconListProcessShow(GtkWidget  *parent)
{
   	RefreshInstallItemId = g_timeout_add (1*1000, refresh_install_item, parent);
    sleep(1);
    return;
}
void Show_Message_Box(char *info,GtkWidget *parent)
{
		GtkWidget *dialog;  
		char tip[256];
		strcpy(tip,info);
		dialog = gtk_message_dialog_new(NULL,  
                GTK_DIALOG_MODAL,  
                GTK_MESSAGE_QUESTION,  
		GTK_BUTTONS_OK,  
		"%s",tip);  
   		 gtk_window_set_title(GTK_WINDOW(dialog), _("Warning"));  
		gtk_dialog_run(GTK_DIALOG(dialog));
		gtk_widget_destroy(dialog);  
}
gboolean refresh_install_item(GtkWidget *parent)
{
	float refresh_install_value = 0.0 ;
    int refresh_install_readRet = 0;
	GString *refresh_install_text;
        char info[256];
	if(installWhitelistItemOver == 1)
	{
		
		if (warning_dialog != NULL)
			gtk_widget_destroy(warning_dialog);
    		installWhitelistItemOver =0 ;
		if(InstallSuccessOrNot == 0)
			strcpy(info,_("install complete,please reboot file compress!"));
		else
			strcpy(info,_("Install failed!"));
		Show_Message_Box(info,parent);

		InstallSuccessOrNot = 0;
                
		return 0;
	}

	
 	return 1;

}

void install_lexicon_list_thread(int total_num)
{
   	int ret;
	printf("install thread num is %d",total_num);
    g_thread_new("installlexiconlist",InstallLexiconListThread,NULL);
}
void InstallLexiconListThread()
{
	pthread_mutex_t install_package_mut_buf = PTHREAD_MUTEX_INITIALIZER;
	
	char run_command[512];
	char run_command_content[512];
	char current_file_path[1024];
	

		memset(run_command_content,0,sizeof(run_command_content));
		snprintf(run_command_content,sizeof(run_command_content),"apt-get --assume-yes --force-yes install rar unrar");
		memset(run_command,0,sizeof(run_command));
		snprintf(run_command,sizeof(run_command),"sudo /usr/share/file-compress/InstallRar \"%s\"",run_command_content);


		printf("аװΪ%s\n",run_command);
		system(run_command);
		
		if (check_package_install("rar")==1)
        {
        	printf(":):):):)(:(:(:װɹ!\n");

                        
       	}
		else
		{
			InstallSuccessOrNot = 1;
			printf("----------װʧ\n");
		}
		sleep(2);

     gdk_threads_enter();
     pthread_mutex_lock(&install_package_mut_buf);
	 printf("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n");
     installWhitelistItemOver =1;
      pthread_mutex_unlock(&install_package_mut_buf);
      gdk_threads_leave();
	  printf("========ˢ°װ\n");

}
void cmd_shell_result(const char* command,char * result,int arrayLen)
{
        FILE *fpRead = NULL;
    char tmp[1024] = "";
        unsigned int count = 0;

        memset(result, 0, arrayLen);

        fpRead = popen(command, "r");
    if (NULL == fpRead)
                return ;

        while(NULL != fgets(tmp,1024-1,fpRead))

        {
                snprintf(result + count, arrayLen - count, "%s", tmp);
                count += strlen(tmp);
                if (count >= arrayLen)
                {
                        printf("count = %d\n", count);
                        printf("tmp = %s\n", tmp);
                        printf("warnning the result is not complete\n");

                        break;
                }
        }
        if (NULL != fpRead)
                pclose(fpRead);
        return ;
}

int check_package_install(char *package_name)
{
        char command_result[10];
    char command_string[1024];

        memset(command_string,0,sizeof(command_string));
    snprintf(command_string,sizeof(command_string),"dpkg -l %s | tail -n 1 |awk '{print $1}'",package_name);
    printf("еΪ%s\n",command_string);

    cmd_shell_result(command_string,command_result,10);
    command_result[strlen(command_result)-1]='\0';

        if (strcmp(command_result,"ii")==0)
        {
//Ѱװ
                return 1;
        }
        else
        {
//δװ
                return 0;
        }

}


static void
file_buffer_ready_cb (GObject      *source_object,
		      GAsyncResult *result,
		      gpointer      user_data)
{
	InstallerData *idata = user_data;
	GFile         *file;
	char          *buffer;
	gsize          buffer_size;
	GError        *error = NULL;
	char          *uri;
	const char    *mime_type;
	gboolean       result_uncertain;
	GType          archive_type;
	FrArchive     *preferred_archive;

	file = G_FILE (source_object);
	if (! _g_file_load_buffer_finish (file, result, &buffer, &buffer_size, &error)) {
		package_installer_terminated (idata, FR_ERROR_GENERIC, error->message);
		g_error_free (error);
		return;
	}

	uri = g_file_get_uri (file);
	mime_type = g_content_type_guess (uri, (guchar *) buffer, buffer_size, &result_uncertain);
	if (result_uncertain) {
		mime_type = _g_mime_type_get_from_content (buffer, buffer_size);
		if (mime_type == NULL)
			mime_type = _g_mime_type_get_from_filename (file);
	}

	g_free (uri);
	g_free (buffer);

	archive_type = get_preferred_archive_for_mime_type (mime_type, FR_ARCHIVE_CAN_READ_WRITE);
	if (archive_type == 0)
		archive_type = get_preferred_archive_for_mime_type (mime_type, FR_ARCHIVE_CAN_READ);
	if (archive_type == 0) {
		if(strcmp(mime_type,"application/vnd.rar")==0)
		{
			printf("rar file\n");
                gchar      *secondary_text;

		secondary_text = g_strdup_printf (_("There is no command installed for %s files.\nDo you want to install rar  to open this file?"),
						  g_content_type_get_description (mime_type));
		GtkWidget *dialog;  
		dialog = gtk_message_dialog_new(GTK_WINDOW(idata->window),  
                GTK_DIALOG_MODAL,  
                GTK_MESSAGE_QUESTION,  
		GTK_BUTTONS_YES_NO,  
		_("There is no command to open this file.Do you want to install rar to open this file?"));  
   		 gtk_window_set_title(GTK_WINDOW(dialog), _("Warning"));  
		 if(gtk_dialog_run(GTK_DIALOG(dialog))==GTK_RESPONSE_YES)
		 {
			 show_warning_dialog(idata->window);

		 }
    		 gtk_widget_destroy(dialog);  
		 return;

		}
		else
		{
		package_installer_terminated (idata, FR_ERROR_GENERIC, _("Archive type not supported."));
		return;
		}
	}
	preferred_archive = g_object_new (archive_type, 0);
	idata->packages = fr_archive_get_packages (preferred_archive, mime_type);
	g_object_unref (preferred_archive);

	if (idata->packages == NULL) {
		package_installer_terminated (idata, FR_ERROR_GENERIC, _("Archive type not supported."));
		return;
	}

#ifdef ENABLE_PACKAGEKIT

	{
		char      *secondary_text;
		GtkWidget *dialog;

		secondary_text = g_strdup_printf (_("There is no command installed for %s files.\nDo you want to search for a command to open this file?"),
						  g_content_type_get_description (mime_type));
		dialog = _gtk_message_dialog_new (GTK_WINDOW (idata->window),
						  GTK_DIALOG_MODAL,
						  _GTK_ICON_NAME_DIALOG_ERROR,
						  _("Could not open this file type"),
						  secondary_text,
						  _GTK_LABEL_CANCEL, GTK_RESPONSE_NO,
						  _("_Search Command"), GTK_RESPONSE_YES,
						  NULL);
		g_signal_connect (dialog, "response", G_CALLBACK (confirm_search_dialog_response_cb), idata);
		gtk_widget_show (dialog);

		g_free (secondary_text);
	}

#else /* ! ENABLE_PACKAGEKIT */

	package_installer_terminated (idata, FR_ERROR_GENERIC, _("Archive type not supported."));

#endif /* ENABLE_PACKAGEKIT */
}


void
dlg_package_installer (FrWindow     *window,
		       GFile        *file,
		       FrAction      action,
		       GCancellable *cancellable)
{
	InstallerData   *idata;

	idata = g_new0 (InstallerData, 1);
	idata->window = g_object_ref (window);
	idata->action = action;
	idata->cancellable = _g_object_ref (cancellable);

	_g_file_load_buffer_async (file,
				   BUFFER_SIZE_FOR_PRELOAD,
				   cancellable,
				   file_buffer_ready_cb,
				   idata);
}
